#!/usr/bin/perl -w
use strict;
$|=1;

my $opt_d = 'dbo';
my $opt_v = 0;

my ($opt_D, $opt_T, $opt_C,$opt_W);
use Getopt::Long qw(:config no_ignore_case bundling);
GetOptions(
	'T'   => \$opt_T, # tables-only mode, exit on INSERT
	'D'   => \$opt_D, # generate full drops
	'C'   => \$opt_C, # generate commits
	'W'   => \$opt_W, # Without Create Table
	'v=i' => \$opt_v, # verbose
	'd=s' => \$opt_d, # db name
	);

my $state='idle';
my $header=0;
my @stack = ();
while (<>) {
	chomp;
	unless ($state =~ /^ins[23]$/) {
		s/\s+/ /g;
		s/(^ | $)//g;
		next unless $_;
	}
	
	if ($opt_W) {
	
		if (/CREATE TABLE/   ) {
			$header=1;
		}

		if ($header  ) {
			#$state = 'm00';
			if (/;/   ) {
				$header=0;
			}
		}

	}
	
		if (! $header) {

# DATE fixups
 	if ('ins2' eq $state) {
		#'12/13/2005 6:18:00 PM'
		# '1/1/1900'
		my $q = "'";
		s/$q([1-9]|1[0-2])\/([1-9]|[12]\d|3[01])\/(\d{4})([$q ])/sprintf("'%04i-%02i-%02i%s",$3,$1,$2,$4)/eg;
		s/ (\d|1[01])(:[0-5]\d:[0-5]\d) ([AP]M)([$q])/sprintf(" %02i%s'",$1+($3 eq 'PM' ? 12 : 0),$2,$3)/eg; 
	}

	if (/^--( |$)/) {
		printf STDERR "DBG: %s/comment '%s'\n", $state, $_ if $opt_v>5;

## TABLES
	} elsif ('idle' eq $state && /^CREATE TABLE (dbo\.)?(\w+)(\(?)$/) {
		printf STDERR "DBG: %s/newtable '%s'\n", $state, $2 if $opt_v>1;
		$state=$3?"table2":"table1";
		if ($opt_D) {
			printf "DROP TABLE IF EXISTS %s.%s;\n", $opt_d, $2;
		}
		die "leftover stack" if @stack;
		printf "CREATE TABLE %s.%s (\n", $opt_d, $2;
	} elsif ('table1' eq $state && /^\($/) {
		$state="table2";
	} elsif ('table2' eq $state && /NULL/) { # old export row defs
		s/ IDENTITY\(\d+,\s*\d+\)//;
		s/^\[(\S+)\]/`$1`/;
		s/ COLLATE \S+ / /;
		s/ smalldatetime/ datetime/;
		s/, --.*$//;
		s/,$//;
		s/^(careerID|specialityID) tinyint /$1 tinyint UNSIGNED /i;
		s/^(usage) /`$1` /i;
		s/ money / float /i;
		push @stack, sprintf "\t%s", $_;
	} elsif ('table2' eq $state && /^CONSTRAINT (\w+) PRIMARY KEY CLUSTERED \(([\w'`, ]+)\)$/) {
		s/ CLUSTERED / /;
		push @stack, sprintf "\t%s", $_;
	} elsif ('table2' eq $state && /^(\w+ [\w\(\)]+( [A-Z]+)?)(,|\);)$/) { # new export row defs
		die "style mix!" if @stack;
		s/ID UNKNOWN/ID INTEGER/; # this one needs to be fixed by ccp!
		s/CHAR/VARCHAR/ if / CHAR\((\d+)\)/ && $1 > 255;
		s/(\s)DATE([,\)])/$1DATETIME$2/;
		printf "  %s\n", $_;
		$state="idle" if /\);$/;
	} elsif ('table2' eq $state && /^\)(;)?$/) {
		printf "%s\n);\n\n", join(",\n", @stack);
		@stack = ();
		$state="idle";

	} elsif ($opt_T && /^INSERT/) {
		exit 23;

## INSERTS
	} elsif ('idle' eq $state && /^INSERT INTO (dbo[\._])?(\w+)$/) {
		printf STDERR "DBG: %s/newinsert '%s'\n", $state, $2 if $opt_v>3;
		$state='ins1';
		printf "INSERT INTO %s.%s ", $opt_d, $2;
	} elsif ('ins1' eq $state && /^\(([\w, ]+)\)$/) {
		printf "%s ", $_;
		$state="ins2";
	} elsif ('ins2' eq $state && /^VALUES\(.*\);$/) {
		printf "%s\n", $_;
		$state="idle";
	} elsif ('ins2' eq $state && /^VALUES\(.*[^;]$/) {
		printf "%s\n", $_;
		$state="ins3";
	} elsif ('ins3' eq $state) {
		printf "%s\n", $_;
		$state="idle" if /\);$/;

	} elsif ('idle' eq $state && /^(COMMIT;|GO)$/) {
		print "COMMIT;\n" if $opt_C;

	} elsif (!$header) {

## WTF
	} else {
		printf STDERR "DBG: %s/unknown '%s'\n", $state, $_;
	}


}

}

print "COMMIT;\n" if $opt_C;



